@@ -538,27 +538,25 @@ int main(int argc, char * argv[]) {
     "               compression; quantization steps size for all subbands are\n"
     "               derived from this value. {The default value for 8bit\n"
     "               images is 0.0039}\n"
-    " -reversible   (false) for irreversible; this should be false to perform\n"
-    "               lossy compression using the 9/7 wavelet transform;\n" 
-    "               or true to perform reversible compression, where\n"
-    "               the 5/3 wavelet is employed with lossless compression.\n"
-    " -colour_trans (true) this option employs a color transform, to\n"
-    "               transform RGB color images into the YUV domain.\n"
-    "               This option should not be used with YUV images, because\n"
+    " -reversible   <true | false> If this is 'false', an irreversible or\n"
+    "               lossy compression is employed, using the 9/7 wavelet\n"
+    "               transform; if 'true', a reversible compression is\n"
+    "               performed, where the 5/3 wavelet is used.\n"
+    "               Default value is 'false'.\n"
+    " -colour_trans <true | false> This option employs a color transform,\n"
+    "               to transform RGB color images into the YUV domain.\n"
+    "               This option should NOT be used with YUV images, because\n"
     "               they have already been transformed.\n"
     "               If there are three color components that are\n"
-    "               downsampled by the same amount then the color transform\n"
-    "               can be true or false. This option is also available\n"
-    "               when there are more than three colour components,\n"
-    "               where it is applied to the first three colour\n"
-    "               components.\n"
-    "               it has already been applied to convert the original RGB\n"
-    "               or whatever the original format to YUV.\n"
+    "               downsampled by the same amount then this option can be\n"
+    "               'true' or 'false'. This option is also available when\n"
+    "               there are more than three colour components, where it is\n"
+    "               applied to the first three colour components.\n"
     " -prog_order   (RPCL) is the progression order, and can be one of:\n"
-    "               LRCP, RLCP, RPCL, PCRL, CPRL\n"
+    "               LRCP, RLCP, RPCL, PCRL, CPRL.\n"
     " -block_size   {x,y} (64,64) where x and y are the height and width of\n"
     "               a codeblock. In unix-like environment, { and } must be\n"
-    "               preceded by a ""\\""\n"
+    "               preceded by a ""\\"".\n"
     " -precincts    {x,y},{x,y},...,{x,y} where {x,y} is the precinct size\n"
     "               starting from the coarsest resolution; the last precinct\n"
     "               is repeated for all finer resolutions\n"
@@ -568,14 +566,15 @@ int main(int argc, char * argv[]) {
     " -tileparts    (None) employ tilepart divisions at each resolution, \n"
     "               indicated by the letter R, and/or component, indicated \n"
     "               by the letter C. For both, use \"-tileparts RC\".\n"
-    " -tlm_marker   (false) insert a TLM marker, either \"true\" or \"false\"\n"
+    " -tlm_marker   <true | false> if 'true', a TLM marker is inserted.\n"
+    "               Default value is false.\n"
     " -profile      (None) is the profile, the code will check if the \n"
     "               selected options meet the profile.  Currently only \n"
     "               BROADCAST and IMF are supported.  This automatically \n"
     "               sets tlm_marker to true and tileparts to C.\n"
     " -com          (None) if set, inserts a COM marker with the specified\n"
     "               string. If the string has spaces, please use\n"
-    "               double quotes, as in -com \"This is a comment\"\n"
+    "               double quotes, as in -com \"This is a comment\".\n"
     "\n"
 
     "When the input file is a YUV file, these arguments need to be \n"
